# - Config file for the Palisade package
# It defines the following variables
#  PALISADE_INCLUDE_DIRS - include directories for Palisade
#  PALISADE_LIBRARIES    - libraries to link against

get_filename_component(PALISADE_CMAKE_DIR "${CMAKE_CURRENT_LIST_FILE}" PATH)

# Our library dependencies (contains definitions for IMPORTED targets)
if(NOT Palisade_BINARY_DIR)
  include("${PALISADE_CMAKE_DIR}/PalisadeTargets.cmake")
endif()

# These are IMPORTED targets created by PalisadeTargets.cmake
set(PALISADE_INCLUDE "/usr/include/palisade")
set(PALISADE_LIBDIR "/usr/lib64")
set(PALISADE_LIBRARIES PALISADEcore;PALISADEpke;PALISADEbinfhe HEXL::hexl -fopenmp)
set(PALISADE_STATIC_LIBRARIES  HEXL::hexl -fopenmp)
set(PALISADE_SHARED_LIBRARIES PALISADEcore;PALISADEpke;PALISADEbinfhe HEXL::hexl -fopenmp)
set(BASE_PALISADE_VERSION 1.11.6)


set(OPENMP_INCLUDES "" )
set(OPENMP_LIBRARIES "" )

set(PALISADE_CXX_FLAGS "-O2 -flto=auto -ffat-lto-objects -fexceptions -g -grecord-gcc-switches -pipe -Wall -Werror=format-security -Wp,-D_FORTIFY_SOURCE=2 -Wp,-D_GLIBCXX_ASSERTIONS -specs=/usr/lib/rpm/anolis/anolis-hardened-cc1 -fstack-protector-strong -specs=/usr/lib/rpm/anolis/anolis-annobin-cc1  -m64 -march=x86-64-v2 -mtune=generic -fasynchronous-unwind-tables -fstack-clash-protection -fcf-protection -Wall -O3  -DPALISADE_VERSION=1.11.6 -Wno-parentheses -DMATHBACKEND=2 -fopenmp -fopenmp")
set(PALISADE_C_FLAGS "-O2 -flto=auto -ffat-lto-objects -fexceptions -g -grecord-gcc-switches -pipe -Wall -Werror=format-security -Wp,-D_FORTIFY_SOURCE=2 -Wp,-D_GLIBCXX_ASSERTIONS -specs=/usr/lib/rpm/anolis/anolis-hardened-cc1 -fstack-protector-strong -specs=/usr/lib/rpm/anolis/anolis-annobin-cc1  -m64 -march=x86-64-v2 -mtune=generic -fasynchronous-unwind-tables -fstack-clash-protection -fcf-protection -Wall -O3  -DPALISADE_VERSION=1.11.6 -DMATHBACKEND=2 -fopenmp -fopenmp")

if( "OFF" STREQUAL "Y" )
	set(PALISADE_CXX_FLAGS "${PALISADE_CXX_FLAGS} -DWITH_NTL" )
	set(PALISADE_C_FLAGS "${PALISADE_C_FLAGS} -DWITH_NTL")
endif()

set (PALISADE_EXE_LINKER_FLAGS "-Wl,-z,relro -Wl,--as-needed  -Wl,-z,now -specs=/usr/lib/rpm/anolis/anolis-hardened-ld -specs=/usr/lib/rpm/anolis/anolis-annobin-cc1  -Wl,--build-id=sha1  ")

# CXX info
set(PALISADE_CXX_STANDARD "11")
set(PALISADE_CXX_COMPILER_ID "GNU")
set(PALISADE_CXX_COMPILER_VERSION "12.1.0")

# Build Options
set(PALISADE_STATIC "OFF")
set(PALISADE_SHARED "ON")
set(PALISADE_TCM "OFF")
set(PALISADE_WITH_INTEL_HEXL "ON")
set(PALISADE_OPENMP "ON")
set(PALISADE_NATIVE_SIZE "64")
set(PALISADE_CKKS_M_FACTOR "1")
set(PALISADE_NATIVEOPT "OFF")

# Math Backend
if("ON")
	set(PALISADE_BACKEND "BE2")
elseif("ON")
	set(PALISADE_BACKEND "BE4")
elseif("OFF")
	set(PALISADE_BACKEND "NTL")
endif()

# Build Details
set(PALISADE_EMSCRIPTEN "")
set(PALISADE_ARCHITECTURE "x86_64")
set(PALISADE_BACKEND_FLAGS_BASE "-DMATHBACKEND=2")

# Compile Definitions

if( "ON" )
	set(PALISADE_BINFHE_COMPILE_DEFINITIONS "_compile_defs-NOTFOUND")
	set(PALISADE_CORE_COMPILE_DEFINITIONS "_compile_defs-NOTFOUND")
	set(PALISADE_PKE_COMPILE_DEFINITIONS "_compile_defs-NOTFOUND")
	set(PALISADE_COMPILE_DEFINITIONS
			${PALISADE_BINFHE_COMPILE_DEFINITIONS}
			${PALISADE_CORE_COMPILE_DEFINITIONS}
			${PALISADE_PKE_COMPILE_DEFINITIONS})
endif()

if( "OFF" )
	set(PALISADE_BINFHE_COMPILE_DEFINITIONS_STATIC "")
	set(PALISADE_CORE_COMPILE_DEFINITIONS_STATIC "")
	set(PALISADE_PKE_COMPILE_DEFINITIONS_STATIC "")
	set(PALISADE_COMPILE_DEFINITIONS_STATIC
			${PALISADE_BINFHE_COMPILE_DEFINITIONS_STATIC}
			${PALISADE_CORE_COMPILE_DEFINITIONS_STATIC}
			${PALISADE_PKE_COMPILE_DEFINITIONS_STATIC})
endif()
